@@ -38,6 +38,10 @@
 #include "dix/input_priv.h"
 #include "dix/dix_priv.h"
 #include "dix/exevents_priv.h"
+#include "dix/inpututils_priv.h"
+#include "dix/request_priv.h"
+#include "dix/screensaver_priv.h"
+#include "dix/window_priv.h"
 #include "mi/mi_priv.h"
 #include "mi/mipointer_priv.h"
 #include "miext/extinit_priv.h"
@@ -60,7 +64,6 @@
 #include "mipointer.h"
 #include "xserver-properties.h"
 #include "eventstr.h"
-#include "inpututils.h"
 
 Bool noTestExtensions = FALSE;
 
@@ -89,35 +92,32 @@ static int XTestSwapFakeInput(ClientPtr /* client */ ,
 static int
 ProcXTestGetVersion(ClientPtr client)
 {
-    xXTestGetVersionReply rep = {
-        .type = X_Reply,
-        .sequenceNumber = client->sequence,
-        .length = 0,
+    X_REQUEST_HEAD_STRUCT(xXTestGetVersionReq);
+    X_REQUEST_FIELD_CARD16(minorVersion);
+
+    xXTestGetVersionReply reply = {
         .majorVersion = XTestMajorVersion,
         .minorVersion = XTestMinorVersion
     };
 
-    REQUEST_SIZE_MATCH(xXTestGetVersionReq);
+    if (client->swapped)
+        swaps(&reply.minorVersion);
 
-    if (client->swapped) {
-        swaps(&rep.sequenceNumber);
-        swaps(&rep.minorVersion);
-    }
-    WriteToClient(client, sizeof(xXTestGetVersionReply), &rep);
-    return Success;
+    return X_SEND_REPLY_SIMPLE(client, reply);
 }
 
 static int
 ProcXTestCompareCursor(ClientPtr client)
 {
-    REQUEST(xXTestCompareCursorReq);
-    xXTestCompareCursorReply rep;
+    X_REQUEST_HEAD_STRUCT(xXTestCompareCursorReq);
+    X_REQUEST_FIELD_CARD32(window);
+    X_REQUEST_FIELD_CARD32(cursor);
+
     WindowPtr pWin;
     CursorPtr pCursor;
     int rc;
     DeviceIntPtr ptr = PickPointer(client);
 
-    REQUEST_SIZE_MATCH(xXTestCompareCursorReq);
     rc = dixLookupWindow(&pWin, stuff->window, client, DixGetAttrAccess);
     if (rc != Success)
         return rc;
@@ -137,17 +137,12 @@ ProcXTestCompareCursor(ClientPtr client)
             return rc;
         }
     }
-    rep = (xXTestCompareCursorReply) {
-        .type = X_Reply,
-        .sequenceNumber = client->sequence,
-        .length = 0,
+
+    xXTestCompareCursorReply reply = {
         .same = (wCursor(pWin) == pCursor)
     };
-    if (client->swapped) {
-        swaps(&rep.sequenceNumber);
-    }
-    WriteToClient(client, sizeof(xXTestCompareCursorReply), &rep);
-    return Success;
+
+    return X_SEND_REPLY_SIMPLE(client, reply);
 }
 
 void
@@ -182,7 +177,14 @@ XTestDeviceSendEvents(DeviceIntPtr dev,
 static int
 ProcXTestFakeInput(ClientPtr client)
 {
-    REQUEST(xXTestFakeInputReq);
+    X_REQUEST_HEAD_NO_CHECK(xXTestFakeInputReq);
+
+    if (client->swapped) {
+        int n = XTestSwapFakeInput(client, (xReq *)stuff);
+        if (n != Success)
+            return n;
+    }
+
     int nev, n, type, rc;
     xEvent *ev;
     DeviceIntPtr dev = NULL;
@@ -348,6 +350,10 @@ ProcXTestFakeInput(ClientPtr client)
             return BadAccess;
 
         dev = GetXTestDevice(dev);
+
+        /* This can only happen if we passed a slave to GetXTestDevice() */
+        if (!dev)
+            return BadAccess;
     }
 
 
@@ -449,9 +455,8 @@ ProcXTestFakeInput(ClientPtr client)
 static int
 ProcXTestGrabControl(ClientPtr client)
 {
-    REQUEST(xXTestGrabControlReq);
+    X_REQUEST_HEAD_STRUCT(xXTestGrabControlReq);
 
-    REQUEST_SIZE_MATCH(xXTestGrabControlReq);
     if ((stuff->impervious != xTrue) && (stuff->impervious != xFalse)) {
         client->errorValue = stuff->impervious;
         return BadValue;
@@ -481,25 +486,6 @@ ProcXTestDispatch(ClientPtr client)
     }
 }
 
-static int _X_COLD
-SProcXTestGetVersion(ClientPtr client)
-{
-    REQUEST(xXTestGetVersionReq);
-    REQUEST_SIZE_MATCH(xXTestGetVersionReq);
-    swaps(&stuff->minorVersion);
-    return ProcXTestGetVersion(client);
-}
-
-static int _X_COLD
-SProcXTestCompareCursor(ClientPtr client)
-{
-    REQUEST(xXTestCompareCursorReq);
-    REQUEST_SIZE_MATCH(xXTestCompareCursorReq);
-    swapl(&stuff->window);
-    swapl(&stuff->cursor);
-    return ProcXTestCompareCursor(client);
-}
-
 static int _X_COLD
 XTestSwapFakeInput(ClientPtr client, xReq * req)
 {
@@ -524,37 +510,6 @@ XTestSwapFakeInput(ClientPtr client, xReq * req)
     return Success;
 }
 
-static int _X_COLD
-SProcXTestFakeInput(ClientPtr client)
-{
-    int n;
-
-    REQUEST(xReq);
-
-    n = XTestSwapFakeInput(client, stuff);
-    if (n != Success)
-        return n;
-    return ProcXTestFakeInput(client);
-}
-
-static int _X_COLD
-SProcXTestDispatch(ClientPtr client)
-{
-    REQUEST(xReq);
-    switch (stuff->data) {
-    case X_XTestGetVersion:
-        return SProcXTestGetVersion(client);
-    case X_XTestCompareCursor:
-        return SProcXTestCompareCursor(client);
-    case X_XTestFakeInput:
-        return SProcXTestFakeInput(client);
-    case X_XTestGrabControl:
-        return ProcXTestGrabControl(client);
-    default:
-        return BadRequest;
-    }
-}
-
 /**
  * Allocate an virtual slave device for xtest events, this
  * is a slave device to inputInfo master devices
@@ -686,7 +641,7 @@ void
 XTestExtensionInit(void)
 {
     AddExtension(XTestExtensionName, 0, 0,
-                 ProcXTestDispatch, SProcXTestDispatch,
+                 ProcXTestDispatch, ProcXTestDispatch,
                  XTestExtensionTearDown, StandardMinorOpcode);
 
     xtest_evlist = InitEventList(GetMaximumEventsNum());
