@@ -35,6 +35,10 @@
 #include "dix/eventconvert.h"
 #include "dix/extension_priv.h"
 #include "dix/input_priv.h"
+#include "dix/inpututils_priv.h"
+#include "dix/screenint_priv.h"
+#include "dix/window_priv.h"
+#include "include/extinit.h"
 #include "os/bug_priv.h"
 
 #include "inputstr.h"
@@ -43,7 +47,6 @@
 #include "exglobals.h"
 #include "enterleave.h"
 #include "xkbsrv.h"
-#include "inpututils.h"
 
 /**
  * @file
@@ -89,8 +92,6 @@ static WindowPtr FocusWindows[MAXDEVICES];
 static BOOL
 HasPointer(DeviceIntPtr dev, WindowPtr win)
 {
-    int i;
-
     /* FIXME: The enter/leave model does not cater for grabbed devices. For
      * now, a quickfix: if the device about to send an enter/leave event to
      * a window is grabbed, assume there is no pointer in that window.
@@ -100,7 +101,7 @@ HasPointer(DeviceIntPtr dev, WindowPtr win)
     if (dev->deviceGrab.grab)
         return FALSE;
 
-    for (i = 0; i < MAXDEVICES; i++)
+    for (int i = 0; i < MAXDEVICES; i++)
         if (PointerWindows[i] == win)
             return TRUE;
 
@@ -114,9 +115,7 @@ HasPointer(DeviceIntPtr dev, WindowPtr win)
 static BOOL
 HasFocus(WindowPtr win)
 {
-    int i;
-
-    for (i = 0; i < MAXDEVICES; i++)
+    for (int i = 0; i < MAXDEVICES; i++)
         if (FocusWindows[i] == win)
             return TRUE;
 
@@ -142,9 +141,7 @@ PointerWin(DeviceIntPtr dev)
 static WindowPtr
 FirstPointerChild(WindowPtr win)
 {
-    int i;
-
-    for (i = 0; i < MAXDEVICES; i++) {
+    for (int i = 0; i < MAXDEVICES; i++) {
         if (PointerWindows[i] && WindowIsParent(win, PointerWindows[i]))
             return PointerWindows[i];
     }
@@ -162,9 +159,7 @@ FirstPointerChild(WindowPtr win)
 static WindowPtr
 FirstFocusChild(WindowPtr win)
 {
-    int i;
-
-    for (i = 0; i < MAXDEVICES; i++) {
+    for (int i = 0; i < MAXDEVICES; i++) {
         if (FocusWindows[i] && FocusWindows[i] != PointerRootWin &&
             WindowIsParent(win, FocusWindows[i]))
             return FocusWindows[i];
@@ -287,12 +282,10 @@ static void
 CoreLeaveNotifies(DeviceIntPtr dev,
                   WindowPtr child, WindowPtr ancestor, int mode, int detail)
 {
-    WindowPtr win;
-
     if (ancestor == child)
         return;
 
-    for (win = child->parent; win != ancestor; win = win->parent) {
+    for (WindowPtr win = child->parent; win != ancestor; win = win->parent) {
         /*Case 7:
            A is a descendant of W, B is above W
 
@@ -332,11 +325,9 @@ DeviceLeaveNotifies(DeviceIntPtr dev,
                     int sourceid,
                     WindowPtr child, WindowPtr ancestor, int mode, int detail)
 {
-    WindowPtr win;
-
     if (ancestor == child)
         return;
-    for (win = child->parent; win != ancestor; win = win->parent) {
+    for (WindowPtr win = child->parent; win != ancestor; win = win->parent) {
         DeviceEnterLeaveEvent(dev, sourceid, XI_Leave, mode, detail, win,
                               child->drawable.id);
         child = win;
@@ -772,7 +763,7 @@ DeviceFocusEvent(DeviceIntPtr dev, int type, int mode, int detail,
                  WindowPtr pWin)
 {
     deviceFocus event;
-    int btlen, len, i;
+    int btlen, len;
 
     DeviceIntPtr mouse = InputDevIsFloating(dev) ?
             dev : GetMaster(dev, MASTER_POINTER);
@@ -784,8 +775,7 @@ DeviceFocusEvent(DeviceIntPtr dev, int type, int mode, int detail,
     len = sizeof(xXIFocusInEvent) + btlen * 4;
 
     xXIFocusInEvent *xi2event = calloc(1, len);
-    if (!xi2event)
-        return;
+    BUG_RETURN(xi2event == NULL);
 
     xi2event->type = GenericEvent;
     xi2event->extension = EXTENSION_MAJOR_XINPUT;
@@ -800,7 +790,7 @@ DeviceFocusEvent(DeviceIntPtr dev, int type, int mode, int detail,
     xi2event->root_x = double_to_fp1616(mouse->spriteInfo->sprite->hot.x);
     xi2event->root_y = double_to_fp1616(mouse->spriteInfo->sprite->hot.y);
 
-    for (i = 0; mouse && mouse->button && i < mouse->button->numButtons; i++)
+    for (int i = 0; mouse && mouse->button && i < mouse->button->numButtons; i++)
         if (BitIsOn(mouse->button->down, i))
             SetBit(&xi2event[1], mouse->button->map[i]);
 
@@ -817,7 +807,7 @@ DeviceFocusEvent(DeviceIntPtr dev, int type, int mode, int detail,
     }
 
     FixUpEventFromWindow(dev->spriteInfo->sprite, (xEvent *) xi2event, pWin,
-                         None, FALSE);
+                         None, FALSE, XI2);
 
     DeliverEventsToWindow(dev, pWin, (xEvent *) xi2event, 1,
                           GetEventFilter(dev, (xEvent *) xi2event), NullGrab);
@@ -849,11 +839,9 @@ static void
 DeviceFocusOutEvents(DeviceIntPtr dev,
                      WindowPtr child, WindowPtr ancestor, int mode, int detail)
 {
-    WindowPtr win;
-
     if (ancestor == child)
         return;
-    for (win = child->parent; win != ancestor; win = win->parent)
+    for (WindowPtr win = child->parent; win != ancestor; win = win->parent)
         DeviceFocusEvent(dev, XI_FocusOut, mode, detail, win);
 }
 
@@ -915,12 +903,10 @@ static void
 CoreFocusOutEvents(DeviceIntPtr dev,
                    WindowPtr child, WindowPtr ancestor, int mode, int detail)
 {
-    WindowPtr win;
-
     if (ancestor == child)
         return;
 
-    for (win = child->parent; win != ancestor; win = win->parent) {
+    for (WindowPtr win = child->parent; win != ancestor; win = win->parent) {
         /*Case 7:
            A is a descendant of W, B is above W
 
@@ -1232,15 +1218,47 @@ CoreFocusToDescendant(DeviceIntPtr dev, WindowPtr A, WindowPtr B, int mode)
 static BOOL
 HasOtherPointer(WindowPtr win, DeviceIntPtr exclude)
 {
-    int i;
-
-    for (i = 0; i < MAXDEVICES; i++)
+    for (int i = 0; i < MAXDEVICES; i++)
         if (i != exclude->id && PointerWindows[i] == win)
             return TRUE;
 
     return FALSE;
 }
 
+/**
+ * Focus moves from PointerRoot to None or from None to PointerRoot.
+ * Assumption: Neither A nor B are valid windows.
+ */
+static void CoreFocusPointerRootNoneSwitchScr(
+    ScreenPtr pScreen,
+    DeviceIntPtr dev,
+    WindowPtr A,     /* PointerRootWin or NoneWin */
+    WindowPtr B,     /* NoneWin or PointerRootWin */
+    int mode)
+{
+    WindowPtr root = pScreen->root;
+
+    if (HasOtherPointer(root, GetMaster(dev, POINTER_OR_FLOAT)) ||
+        FirstFocusChild(root))
+        return;
+
+    /* If pointer was on PointerRootWin and changes to NoneWin, and
+     * the pointer paired with dev is below the current root window,
+     * do a NotifyPointer run. */
+    if (dev->focus && dev->focus->win == PointerRootWin &&
+        B != PointerRootWin) {
+        WindowPtr ptrwin = PointerWin(GetMaster(dev, POINTER_OR_FLOAT));
+            if (ptrwin && WindowIsParent(root, ptrwin))
+            CoreFocusOutNotifyPointerEvents(dev, root, None, mode, TRUE);
+    }
+    CoreFocusEvent(dev, FocusOut, mode,
+                   A ? NotifyPointerRoot : NotifyDetailNone, root);
+    CoreFocusEvent(dev, FocusIn, mode,
+                   B ? NotifyPointerRoot : NotifyDetailNone, root);
+    if (B == PointerRootWin)
+        CoreFocusInNotifyPointerEvents(dev, root, None, mode, TRUE);
+}
+
 /**
  * Focus moves from PointerRoot to None or from None to PointerRoot.
  * Assumption: Neither A nor B are valid windows.
@@ -1251,39 +1269,31 @@ CoreFocusPointerRootNoneSwitch(DeviceIntPtr dev,
                                WindowPtr B,     /* NoneWin or PointerRootWin */
                                int mode)
 {
-    WindowPtr root;
-    int i;
-    int nscreens = screenInfo.numScreens;
+    DIX_FOR_EACH_SCREEN_XINERAMA({
+        CoreFocusPointerRootNoneSwitchScr(walkScreen, dev, A, B, mode);
+    });
+}
 
-#ifdef XINERAMA
-    if (!noPanoramiXExtension)
-        nscreens = 1;
-#endif /* XINERAMA */
+/**
+ * Focus moves from window A to PointerRoot or to None.
+ * Assumption: A is a valid window and not PointerRoot or None.
+ */
+static void CoreFocusToPointerRootOrNoneScr(
+    ScreenPtr pScreen,
+    DeviceIntPtr dev,
+    WindowPtr A,
+    WindowPtr B,        /* PointerRootWin or NoneWin */
+    int mode)
+{
+    WindowPtr root = pScreen->root;
 
-    for (i = 0; i < nscreens; i++) {
-        root = screenInfo.screens[i]->root;
-        if (!HasOtherPointer(root, GetMaster(dev, POINTER_OR_FLOAT)) &&
-            !FirstFocusChild(root)) {
-            /* If pointer was on PointerRootWin and changes to NoneWin, and
-             * the pointer paired with dev is below the current root window,
-             * do a NotifyPointer run. */
-            if (dev->focus && dev->focus->win == PointerRootWin &&
-                B != PointerRootWin) {
-                WindowPtr ptrwin = PointerWin(GetMaster(dev, POINTER_OR_FLOAT));
+    if (HasFocus(root) || FirstFocusChild(root))
+        return;
 
-                if (ptrwin && WindowIsParent(root, ptrwin))
-                    CoreFocusOutNotifyPointerEvents(dev, root, None, mode,
-                                                    TRUE);
-            }
-            CoreFocusEvent(dev, FocusOut, mode,
-                           A ? NotifyPointerRoot : NotifyDetailNone, root);
-            CoreFocusEvent(dev, FocusIn, mode,
-                           B ? NotifyPointerRoot : NotifyDetailNone, root);
-            if (B == PointerRootWin)
-                CoreFocusInNotifyPointerEvents(dev, root, None, mode, TRUE);
-        }
-
-    }
+    CoreFocusEvent(dev, FocusIn, mode,
+                   B ? NotifyPointerRoot : NotifyDetailNone, root);
+    if (B == PointerRootWin)
+        CoreFocusInNotifyPointerEvents(dev, root, None, mode, TRUE);
 }
 
 /**
@@ -1295,15 +1305,6 @@ CoreFocusToPointerRootOrNone(DeviceIntPtr dev, WindowPtr A,
                              WindowPtr B,        /* PointerRootWin or NoneWin */
                              int mode)
 {
-    WindowPtr root;
-    int i;
-    int nscreens = screenInfo.numScreens;
-
-#ifdef XINERAMA
-    if (!noPanoramiXExtension)
-        nscreens = 1;
-#endif /* XINERAMA */
-
     if (!HasFocus(A)) {
         WindowPtr child = FirstFocusChild(A);
 
@@ -1322,15 +1323,37 @@ CoreFocusToPointerRootOrNone(DeviceIntPtr dev, WindowPtr A,
     /* NullWindow means we include the root window */
     CoreFocusOutEvents(dev, A, NullWindow, mode, NotifyNonlinearVirtual);
 
-    for (i = 0; i < nscreens; i++) {
-        root = screenInfo.screens[i]->root;
-        if (!HasFocus(root) && !FirstFocusChild(root)) {
-            CoreFocusEvent(dev, FocusIn, mode,
-                           B ? NotifyPointerRoot : NotifyDetailNone, root);
-            if (B == PointerRootWin)
-                CoreFocusInNotifyPointerEvents(dev, root, None, mode, TRUE);
-        }
+    DIX_FOR_EACH_SCREEN_XINERAMA({
+        CoreFocusToPointerRootOrNoneScr(walkScreen, dev, A, B, mode);
+    });
+}
+
+/**
+ * Focus moves from PointerRoot or None to a window B.
+ * Assumption: B is a valid window and not PointerRoot or None.
+ */
+static void CoreFocusFromPointerRootOrNoneScr(
+    ScreenPtr pScreen,
+    DeviceIntPtr dev,
+    WindowPtr A,   /* PointerRootWin or NoneWin */
+    WindowPtr B, int mode)
+{
+    WindowPtr root = pScreen->root;
+
+    if (HasFocus(root) || FirstFocusChild(root))
+        return;
+
+    /* If pointer was on PointerRootWin and changes to NoneWin, and
+     * the pointer paired with dev is below the current root window,
+     * do a NotifyPointer run. */
+    if (dev->focus && dev->focus->win == PointerRootWin &&
+        B != PointerRootWin) {
+        WindowPtr ptrwin = PointerWin(GetMaster(dev, POINTER_OR_FLOAT));
+        if (ptrwin)
+            CoreFocusOutNotifyPointerEvents(dev, root, None, mode, TRUE);
     }
+    CoreFocusEvent(dev, FocusOut, mode,
+                   A ? NotifyPointerRoot : NotifyDetailNone, root);
 }
 
 /**
@@ -1342,35 +1365,11 @@ CoreFocusFromPointerRootOrNone(DeviceIntPtr dev,
                                WindowPtr A,   /* PointerRootWin or NoneWin */
                                WindowPtr B, int mode)
 {
-    WindowPtr root;
-    int i;
-    int nscreens = screenInfo.numScreens;
+    DIX_FOR_EACH_SCREEN_XINERAMA({
+        CoreFocusFromPointerRootOrNoneScr(walkScreen, dev, A, B, mode);
+    });
 
-#ifdef XINERAMA
-    if (!noPanoramiXExtension)
-        nscreens = 1;
-#endif /* XINERAMA */
-
-    for (i = 0; i < nscreens; i++) {
-        root = screenInfo.screens[i]->root;
-        if (!HasFocus(root) && !FirstFocusChild(root)) {
-            /* If pointer was on PointerRootWin and changes to NoneWin, and
-             * the pointer paired with dev is below the current root window,
-             * do a NotifyPointer run. */
-            if (dev->focus && dev->focus->win == PointerRootWin &&
-                B != PointerRootWin) {
-                WindowPtr ptrwin = PointerWin(GetMaster(dev, POINTER_OR_FLOAT));
-
-                if (ptrwin)
-                    CoreFocusOutNotifyPointerEvents(dev, root, None, mode,
-                                                    TRUE);
-            }
-            CoreFocusEvent(dev, FocusOut, mode,
-                           A ? NotifyPointerRoot : NotifyDetailNone, root);
-        }
-    }
-
-    root = B;                   /* get B's root window */
+    WindowPtr root = B;                   /* get B's root window */
     while (root->parent)
         root = root->parent;
 
@@ -1426,8 +1425,6 @@ DeviceFocusEvents(DeviceIntPtr dev, WindowPtr from, WindowPtr to, int mode)
 {
     int out, in;                /* for holding details for to/from
                                    PointerRoot/None */
-    int i;
-    int nscreens = screenInfo.numScreens;
     SpritePtr sprite = dev->spriteInfo->sprite;
 
     if (from == to)
@@ -1436,11 +1433,6 @@ DeviceFocusEvents(DeviceIntPtr dev, WindowPtr from, WindowPtr to, int mode)
     in = (to == NoneWin) ? NotifyDetailNone : NotifyPointerRoot;
     /* wrong values if neither, but then not referenced */
 
-#ifdef XINERAMA
-    if (!noPanoramiXExtension)
-        nscreens = 1;
-#endif /* XINERAMA */
-
     if ((to == NullWindow) || (to == PointerRootWin)) {
         if ((from == NullWindow) || (from == PointerRootWin)) {
             if (from == PointerRootWin) {
@@ -1451,9 +1443,9 @@ DeviceFocusEvents(DeviceIntPtr dev, WindowPtr from, WindowPtr to, int mode)
                                      NotifyPointer);
             }
             /* Notify all the roots */
-            for (i = 0; i < nscreens; i++)
-                DeviceFocusEvent(dev, XI_FocusOut, mode, out,
-                                 screenInfo.screens[i]->root);
+            DIX_FOR_EACH_SCREEN_XINERAMA({
+                DeviceFocusEvent(dev, XI_FocusOut, mode, out, walkScreen->root);
+            });
         }
         else {
             if (WindowIsParent(from, sprite->win)) {
@@ -1467,10 +1459,12 @@ DeviceFocusEvents(DeviceIntPtr dev, WindowPtr from, WindowPtr to, int mode)
             DeviceFocusOutEvents(dev, from, NullWindow, mode,
                                  NotifyNonlinearVirtual);
         }
+
         /* Notify all the roots */
-        for (i = 0; i < nscreens; i++)
-            DeviceFocusEvent(dev, XI_FocusIn, mode, in,
-                             screenInfo.screens[i]->root);
+        DIX_FOR_EACH_SCREEN_XINERAMA({
+            DeviceFocusEvent(dev, XI_FocusIn, mode, in, walkScreen->root);
+        });
+
         if (to == PointerRootWin) {
             DeviceFocusInEvents(dev, InputDevCurrentRootWindow(dev), sprite->win,
                                 mode, NotifyPointer);
@@ -1486,9 +1480,11 @@ DeviceFocusEvents(DeviceIntPtr dev, WindowPtr from, WindowPtr to, int mode)
                                      InputDevCurrentRootWindow(dev), mode,
                                      NotifyPointer);
             }
-            for (i = 0; i < nscreens; i++)
-                DeviceFocusEvent(dev, XI_FocusOut, mode, out,
-                                 screenInfo.screens[i]->root);
+
+            DIX_FOR_EACH_SCREEN_XINERAMA({
+                DeviceFocusEvent(dev, XI_FocusOut, mode, out, walkScreen->root);
+            });
+
             if (to->parent != NullWindow)
                 DeviceFocusInEvents(dev, InputDevCurrentRootWindow(dev), to, mode,
                                     NotifyNonlinearVirtual);
