@@ -39,6 +39,7 @@
 #include <X11/X.h>
 
 #include "config/hotplug_priv.h"
+#include "os/osdep.h"
 
 #include "os.h"
 #include "xf86_priv.h"
@@ -48,6 +49,7 @@
 
 #include "xf86Bus.h"
 #include "xf86sbusBus_priv.h"
+#include "xf86platformBus_priv.h"
 
 #include "xf86_OSproc.h"
 #include "xf86VGAarbiter_priv.h"
@@ -556,30 +558,6 @@ xf86GetDevFromEntity(int entityIndex, int instance)
     return NULL;
 }
 
-/*
- * xf86PostProbe() -- Allocate all non conflicting resources
- * This function gets called by xf86Init().
- */
-void
-xf86PostProbe(void)
-{
-    if (fbSlotClaimed && (
-#if (defined(__sparc__) || defined(__sparc)) && !defined(__OpenBSD__)
-                             sbusSlotClaimed ||
-#endif
-#ifdef XSERVER_PLATFORM_BUS
-                             platformSlotClaimed ||
-#endif
-#ifdef XSERVER_LIBPCIACCESS
-                             pciSlotClaimed
-#else
-                             TRUE
-#endif
-        ))
-        FatalError("Cannot run in framebuffer mode. Please specify busIDs "
-                   "       for all framebuffer devices\n");
-}
-
 Bool
 xf86IsEntityShared(int entityIndex)
 {
@@ -676,3 +654,227 @@ xf86GetEntityPrivate(int entityIndex, int privIndex)
 
     return &(xf86Entities[entityIndex]->entityPrivates[privIndex]);
 }
+
+/*
+ * Check if the slot requested is free.  If it is already in use, return FALSE.
+ */
+
+Bool
+xf86CheckSlot(const void *ptr, BusType type)
+{
+    int i;
+
+#ifdef XSERVER_LIBPCIACCESS
+    const struct pci_device *pci_ptr = (type == BUS_PCI ?
+             (const struct pci_device *)ptr : NULL);
+#endif
+
+#ifdef XSERVER_PLATFORM_BUS
+    const struct xf86_platform_device *plat_ptr = (type == BUS_PLATFORM ?
+             (const struct xf86_platform_device *)ptr : NULL);
+#endif
+
+    GDevPtr fb_ptr = (type == BUS_NONE ?
+             (GDevPtr)ptr : NULL);
+    const char *msPath = NULL;
+    const char *fbPath = NULL;
+
+    if (ptr == NULL) {
+        return FALSE;
+    }
+
+#ifdef XSERVER_PLATFORM_BUS
+    /* XSERVER_PLATFORM_BUS assumes XSERVER_LIBPCIACCESS */
+    if (plat_ptr) {
+        pci_ptr = plat_ptr->pdev;
+        msPath = plat_ptr->attribs->path;
+    }
+#endif
+
+    if (type == BUS_NONE) {
+        if (!strcasecmp(fb_ptr->driver, "modesetting")) {
+   /*
+    * If xf86ClaimFbSlot() is called by modesetting driver,
+    * busID has not been set and the device name was not specified
+    * via "kmsdev" option, the default "/dev/dri/card0" is used.
+    *
+    * We have to check whether a platform device has previously
+    * grabbed the device we are going to claim.
+    */
+            msPath = xf86FindOptionValue(fb_ptr->options, "kmsdev");
+            if (msPath == NULL) {
+                /* Autoconfigured */
+                msPath = "/dev/dri/card0";
+            }
+        }
+        else
+        if (!strcasecmp(fb_ptr->driver, "fbdev")) {
+   /*
+    * fbdev driver can also call xf86ClaimFbSlot() for
+    * an autoconfigured device, or the device name can be set
+    * via "fbdev" option.
+    */
+            fbPath = xf86FindOptionValue(fb_ptr->options, "fbdev");
+            if (fbPath == NULL) {
+                /* Autoconfigured */
+                fbPath = "";
+            }
+        }
+    }
+
+   /*
+    * Having prepared all data about a candidate, we walk
+    * through all previous entities to check for a collision.
+    */
+
+    for (i = 0; i < xf86NumEntities; i++) {
+        const EntityPtr pent = xf86Entities[i];
+#ifdef XSERVER_LIBPCIACCESS
+        struct pci_device *pci_other;
+#endif
+        const char *msOther = NULL;
+        const char *fbOther = NULL;
+
+        if (pent->numInstances <= 0) {
+        /* All devices are unclaimed, ignore this entity */
+            continue;
+        }
+
+        if ((fbPath != NULL) && (*fbPath == '\0')) {
+            /* Autoconfigured fbdev device is incompatible with anything */
+            LogMessageVerb(X_INFO, 1,
+                "\"%s\" must be the only device, but \"%s\" is present.\n",
+                fb_ptr->identifier, pent->devices[0]->identifier);
+            return FALSE;
+        }
+
+#ifdef XSERVER_LIBPCIACCESS 
+        pci_other = xf86GetPciInfoForEntity(i);
+        /* First compare PCI addresses */
+        if (pci_ptr && pci_other) {
+            if (MATCH_PCI_DEVICES(pci_other, pci_ptr)) {
+            /* This PCI slot has been claimed, fail */
+                if (msPath) {
+                    LogMessageVerb(X_INFO, 1,
+                        " Platform device \"%s\" skipped because\n",
+                        msPath);
+                }
+                else {
+                    LogMessageVerb(X_INFO, 1,
+                        " PCI device skipped because\n");
+                }
+                LogMessageVerb(X_INFO, 1,
+                    "  PCI bus id %u@%u:%u:%u has already been claimed by \"%s\".\n",
+                    pci_ptr->domain, pci_ptr->bus, pci_ptr->dev, pci_ptr->func, 
+                    pent->devices[0]->identifier);
+                return FALSE;
+            }
+            else
+            /* This is another device, skip */
+                continue;
+        }
+
+        if (pent->bus.type == BUS_PCI) {
+            /* No other means to compare, accept */
+            continue;
+        }
+#endif
+
+        if (pent->bus.type == BUS_NONE) {
+            if (!strcasecmp(pent->driver->driverName, "fbdev")) {
+                if ((type != BUS_NONE) || (fbPath == NULL)) {
+                    /* fbdev without busID is incompatible with other types */
+                    LogMessageVerb(X_INFO, 1,
+                        " Only fbdev without PCI bus id can be claimed after \"%s\".\n",
+                        pent->devices[0]->identifier);
+                    return FALSE;
+                }
+                /* Examine the first device only */
+                fbOther = xf86FindOptionValue(pent->devices[0]->options, "fbdev");
+                if (fbOther == NULL) {
+                    /* Autoconfigured, reject */
+                    LogMessageVerb(X_INFO, 1,
+                        " Can\'t claim anything after \"%s\".\n",
+                        pent->devices[0]->identifier);
+                    return FALSE;
+                }
+                if (strcmp(fbPath, fbOther)) {
+                    /* No conflict */
+                    continue;
+                }
+                else {
+                    /* This framebuffer device has been claimed already */
+                    LogMessageVerb(X_INFO, 1,
+                        " Framebuffer device \"%s\" has already been claimed by \"%s\".\n",
+                        fbPath, pent->devices[0]->identifier);
+                    return FALSE;
+                }
+            }
+        }
+
+#ifdef XSERVER_PLATFORM_BUS
+        if (pent->bus.type == BUS_PLATFORM) {
+            msOther = pent->bus.id.plat->attribs->path;
+        } else
+#endif
+        if (pent->bus.type == BUS_NONE) {
+            if (!strcasecmp(pent->driver->driverName, "modesetting")) {
+                /* Examine the first device only */
+                msOther = xf86FindOptionValue(pent->devices[0]->options, "kmsdev");
+                if (msOther == NULL)
+#ifdef XSERVER_LIBPCIACCESS
+                    if (pci_other == NULL)
+#endif
+                    /* Autoconfigured */
+                    msOther = "/dev/dri/card0";
+            }
+        }
+
+        if ((msPath != NULL) && (msOther != NULL) && !strcmp(msPath, msOther)) {
+            /* This DRI device has been claimed already */
+                    LogMessageVerb(X_INFO, 1,
+                        " DRI device \"%s\" has already been claimed by \"%s\".\n",
+                        msPath, pent->devices[0]->identifier);
+            return FALSE;
+        }
+    }
+
+#ifdef XSERVER_PLATFORM_BUS
+    if (type == BUS_PLATFORM) {
+        if (pci_ptr)
+            LogMessageVerb(X_INFO, 1,
+                " Platform device \"%s\" at %u@%u:%u:%u can be claimed.\n",
+                msPath, pci_ptr->domain, pci_ptr->bus, pci_ptr->dev, pci_ptr->func);
+        else
+            LogMessageVerb(X_INFO, 1,
+                " Platform device \"%s\" can be claimed.\n",
+                 msPath);
+    }
+    else
+#endif
+#ifdef XSERVER_LIBPCIACCESS 
+    if (type == BUS_PCI) {
+        LogMessageVerb(X_INFO, 1,
+            " PCI device %u@%u:%u:%u can be claimed.\n",
+            pci_ptr->domain, pci_ptr->bus, pci_ptr->dev, pci_ptr->func);
+    }
+    else
+#endif
+    if (type == BUS_NONE) {
+        if (msPath)
+            LogMessageVerb(X_INFO, 1,
+                "\"%s\" can be claimed by modesetting driver as \"%s\".\n",
+                msPath, fb_ptr->identifier);
+        else
+        if (fbPath)
+            LogMessageVerb(X_INFO, 1,
+                "\"%s\" can be claimed by fbdev driver as \"%s\".\n",
+                fbPath, fb_ptr->identifier);
+        else
+            LogMessageVerb(X_INFO, 1,
+                "\"%s\" can be claimed.\n",
+                 fb_ptr->identifier);
+    }
+
+    return TRUE;
+}
