@@ -45,6 +45,8 @@
 
 #include "os/osdep.h"
 
+#include "seatd-libseat.h"
+
 static Bool KeepTty = FALSE;
 
 #if defined (SYSCONS_SUPPORT) || defined (PCVT_SUPPORT)
@@ -134,6 +136,13 @@ static xf86ConsOpen_t xf86ConsTab[] = {
     (xf86ConsOpen_t) NULL
 };
 
+
+Bool
+xf86VTKeepTtyIsSet(void)
+{
+     return KeepTty;
+}
+
 void
 xf86OpenConsole(void)
 {
@@ -150,6 +159,12 @@ xf86OpenConsole(void)
 #endif
 
     if (serverGeneration == 1) {
+
+        /* If libseat is in control, it handles VT switching. */
+        if (seatd_libseat_controls_session()) {
+            return;
+        }
+
         /* check if we are run with euid==0 */
         if (geteuid() != 0) {
             FatalError("xf86OpenConsole: Server must be suid root");
