@@ -34,7 +34,9 @@
 
 #include <errno.h>
 
+#include "dix/dix_priv.h"
 #include "dix/property_priv.h"
+#include "dix/request_priv.h"
 
 #include "quartz.h"
 
@@ -58,11 +60,11 @@
 
 #define DEFINE_ATOM_HELPER(func, atom_name)                      \
     static Atom func(void) {                                       \
-        static int generation;                                      \
+        static x_server_generation_t generation;                    \
         static Atom atom;                                           \
         if (generation != serverGeneration) {                       \
             generation = serverGeneration;                          \
-            atom = MakeAtom(atom_name, strlen(atom_name), TRUE);  \
+            atom = dixAddAtom(atom_name);                           \
         }                                                           \
         return atom;                                                \
     }
@@ -145,21 +147,15 @@ AppleWMDoReorderWindow(WindowPtr pWin)
 static int
 ProcAppleWMQueryVersion(register ClientPtr client)
 {
-    xAppleWMQueryVersionReply rep;
-
     REQUEST_SIZE_MATCH(xAppleWMQueryVersionReq);
-    rep.type = X_Reply;
-    rep.length = 0;
-    rep.sequenceNumber = client->sequence;
-    rep.majorVersion = SERVER_APPLEWM_MAJOR_VERSION;
-    rep.minorVersion = SERVER_APPLEWM_MINOR_VERSION;
-    rep.patchVersion = SERVER_APPLEWM_PATCH_VERSION;
-    if (client->swapped) {
-        swaps(&rep.sequenceNumber);
-        swapl(&rep.length);
-    }
-    WriteToClient(client, sizeof(xAppleWMQueryVersionReply),&rep);
-    return Success;
+
+    xAppleWMQueryVersionReply reply = {
+        .majorVersion = SERVER_APPLEWM_MAJOR_VERSION,
+        .minorVersion = SERVER_APPLEWM_MINOR_VERSION,
+        .patchVersion = SERVER_APPLEWM_PATCH_VERSION,
+    };
+
+    return X_SEND_REPLY_SIMPLE(client, reply);
 }
 
 /* events */
@@ -224,7 +220,7 @@ static int
 ProcAppleWMSelectInput(register ClientPtr client)
 {
     REQUEST(xAppleWMSelectInputReq);
-    WMEventPtr pEvent, pNewEvent, *pHead;
+    WMEventPtr pEvent, *pHead;
     XID clientResource;
     int i;
 
@@ -246,7 +242,7 @@ ProcAppleWMSelectInput(register ClientPtr client)
         }
 
         /* build the entry */
-        pNewEvent = (WMEventPtr)malloc(sizeof(WMEventRec));
+        WMEventPtr pNewEvent = calloc(1, sizeof(WMEventRec));
         if (!pNewEvent)
             return BadAlloc;
         pNewEvent->next = 0;
@@ -267,7 +263,7 @@ ProcAppleWMSelectInput(register ClientPtr client)
          * done through the resource database.
          */
         if (i != Success || !pHead) {
-            pHead = (WMEventPtr *)malloc(sizeof(WMEventPtr));
+            pHead = calloc(1, sizeof(WMEventPtr));
             if (!pHead ||
                 !AddResource(eventResource, EventType, (void *)pHead)) {
                 FreeResource(clientResource, X11_RESTYPE_NONE);
@@ -282,7 +278,7 @@ ProcAppleWMSelectInput(register ClientPtr client)
     else if (stuff->mask == 0) {
         /* delete the interest */
         if (i == Success && pHead) {
-            pNewEvent = 0;
+            WMEventPtr pNewEvent = 0;
             for (pEvent = *pHead; pEvent; pEvent = pEvent->next) {
                 if (pEvent->client == client)
                     break;
@@ -368,16 +364,15 @@ ProcAppleWMReenableUpdate(register ClientPtr client)
 static int
 ProcAppleWMSetWindowMenu(register ClientPtr client)
 {
-    const char *bytes, **items;
-    char *shortcuts;
+    const char *bytes;
     int max_len, nitems, i, j;
     REQUEST(xAppleWMSetWindowMenuReq);
 
     REQUEST_AT_LEAST_SIZE(xAppleWMSetWindowMenuReq);
 
     nitems = stuff->nitems;
-    items = malloc(sizeof(char *) * nitems);
-    shortcuts = malloc(sizeof(char) * nitems);
+    const char **items = calloc(nitems, sizeof(char *));
+    char *shortcuts = calloc(nitems, sizeof(char));
 
     if (!items || !shortcuts) {
         free(items);
@@ -527,14 +522,10 @@ ProcAppleWMSetCanQuit(register ClientPtr client)
 static int
 ProcAppleWMFrameGetRect(register ClientPtr client)
 {
-    xAppleWMFrameGetRectReply rep;
     BoxRec ir, or, rr;
     REQUEST(xAppleWMFrameGetRectReq);
 
     REQUEST_SIZE_MATCH(xAppleWMFrameGetRectReq);
-    rep.type = X_Reply;
-    rep.length = 0;
-    rep.sequenceNumber = client->sequence;
 
     ir = make_box(stuff->ix, stuff->iy, stuff->iw, stuff->ih);
     or = make_box(stuff->ox, stuff->oy, stuff->ow, stuff->oh);
@@ -545,27 +536,24 @@ ProcAppleWMFrameGetRect(register ClientPtr client)
         return BadValue;
     }
 
-    rep.x = rr.x1;
-    rep.y = rr.y1;
-    rep.w = rr.x2 - rr.x1;
-    rep.h = rr.y2 - rr.y1;
+    xAppleWMFrameGetRectReply reply = {
+        .x = rr.x1,
+        .y = rr.y1,
+        .w = rr.x2 - rr.x1,
+        .h = rr.y2 - rr.y1,
+    };
 
-    WriteToClient(client, sizeof(xAppleWMFrameGetRectReply),&rep);
-    return Success;
+    return X_SEND_REPLY_SIMPLE(client, reply);
 }
 
 static int
 ProcAppleWMFrameHitTest(register ClientPtr client)
 {
-    xAppleWMFrameHitTestReply rep;
     BoxRec ir, or;
     int ret;
     REQUEST(xAppleWMFrameHitTestReq);
 
     REQUEST_SIZE_MATCH(xAppleWMFrameHitTestReq);
-    rep.type = X_Reply;
-    rep.length = 0;
-    rep.sequenceNumber = client->sequence;
 
     ir = make_box(stuff->ix, stuff->iy, stuff->iw, stuff->ih);
     or = make_box(stuff->ox, stuff->oy, stuff->ow, stuff->oh);
@@ -575,10 +563,11 @@ ProcAppleWMFrameHitTest(register ClientPtr client)
         return BadValue;
     }
 
-    rep.ret = ret;
+    xAppleWMFrameHitTestReply reply = {
+        .ret = ret
+    };
 
-    WriteToClient(client, sizeof(xAppleWMFrameHitTestReply),&rep);
-    return Success;
+    return X_SEND_REPLY_SIMPLE(client, reply);
 }
 
 static int
@@ -687,31 +676,6 @@ SNotifyEvent(xAppleWMNotifyEvent *from, xAppleWMNotifyEvent *to)
     cpswapl(from->arg, to->arg);
 }
 
-static int
-SProcAppleWMQueryVersion(register ClientPtr client)
-{
-    return ProcAppleWMQueryVersion(client);
-}
-
-static int
-SProcAppleWMDispatch(register ClientPtr client)
-{
-    REQUEST(xReq);
-
-    /* It is bound to be non-local when there is byte swapping */
-    if (!client->local)
-        return WMErrorBase + AppleWMClientNotLocal;
-
-    /* only local clients are allowed WM access */
-    switch (stuff->data) {
-    case X_AppleWMQueryVersion:
-        return SProcAppleWMQueryVersion(client);
-
-    default:
-        return BadRequest;
-    }
-}
-
 void
 AppleWMExtensionInit(AppleWMProcsPtr procsPtr)
 {
@@ -726,7 +690,7 @@ AppleWMExtensionInit(AppleWMProcsPtr procsPtr)
                                  AppleWMNumberEvents,
                                  AppleWMNumberErrors,
                                  ProcAppleWMDispatch,
-                                 SProcAppleWMDispatch,
+                                 ProcAppleWMDispatch,
                                  NULL,
                                  StandardMinorOpcode))) {
         size_t i;
