@@ -265,7 +265,7 @@ CreateSurfaceForWindow(ScreenPtr pScreen, WindowPtr pWin,
         xp_window_changes wc;
 
         /* allocate a DRI Window Private record */
-        if (!(pDRIDrawablePriv = malloc(sizeof(*pDRIDrawablePriv)))) {
+        if (!(pDRIDrawablePriv = calloc(1, sizeof(*pDRIDrawablePriv)))) {
             return NULL;
         }
 
@@ -681,7 +681,6 @@ DRICreatePixmap(ScreenPtr pScreen, Drawable id,
                 DrawablePtr pDrawable, char *path,
                 size_t pathmax)
 {
-    DRIPixmapBufferPtr shared;
     PixmapPtr pPix;
 
     if (pDrawable->type != DRAWABLE_PIXMAP)
@@ -689,7 +688,7 @@ DRICreatePixmap(ScreenPtr pScreen, Drawable id,
 
     pPix = (PixmapPtr)pDrawable;
 
-    shared = malloc(sizeof(*shared));
+    DRIPixmapBufferPtr shared = calloc(1, sizeof(*shared));
     if (NULL == shared) {
         FatalError("failed to allocate DRIPixmapBuffer in %s\n", __func__);
     }
