@@ -47,7 +47,10 @@
 #include "dix/cursor_priv.h"
 #include "dix/dix_priv.h"
 #include "dix/input_priv.h"
+#include "dix/request_priv.h"
+#include "dix/rpcbuf_priv.h"
 #include "dix/screen_hooks_priv.h"
+#include "dix/screenint_priv.h"
 
 #include "xfixesint.h"
 #include "scrnintstr.h"
@@ -261,6 +264,12 @@ ProcXFixesSelectCursorInput(ClientPtr client)
     int rc;
 
     REQUEST_SIZE_MATCH(xXFixesSelectCursorInputReq);
+
+    if (client->swapped) {
+        swapl(&stuff->window);
+        swapl(&stuff->eventMask);
+    }
+
     rc = dixLookupWindow(&pWin, stuff->window, client, DixGetAttrAccess);
     if (rc != Success)
         return rc;
@@ -287,16 +296,6 @@ GetBit(unsigned char *line, int x)
     return 0;
 }
 
-int _X_COLD
-SProcXFixesSelectCursorInput(ClientPtr client)
-{
-    REQUEST(xXFixesSelectCursorInputReq);
-    REQUEST_SIZE_MATCH(xXFixesSelectCursorInputReq);
-    swapl(&stuff->window);
-    swapl(&stuff->eventMask);
-    return ProcXFixesSelectCursorInput(client);
-}
-
 void _X_COLD
 SXFixesCursorNotifyEvent(xXFixesCursorNotifyEvent * from,
                          xXFixesCursorNotifyEvent * to)
@@ -369,16 +368,17 @@ ProcXFixesGetCursorImage(ClientPtr client)
     height = pCursor->bits->height;
     npixels = width * height;
 
-    CARD32 *image = calloc(npixels, sizeof(CARD32));
+    x_rpcbuf_t rpcbuf = { .swapped = client->swapped, .err_clear = TRUE };
+
+    CARD32 *image = x_rpcbuf_reserve(&rpcbuf, npixels * sizeof(CARD32));
     if (!image)
         return BadAlloc;
 
     CopyCursorToImage(pCursor, image);
+    if (client->swapped)
+        SwapLongs(image, npixels);
 
-    xXFixesGetCursorImageReply rep = {
-        .type = X_Reply,
-        .sequenceNumber = client->sequence,
-        .length = npixels,
+    xXFixesGetCursorImageReply reply = {
         .width = width,
         .height = height,
         .x = x,
@@ -389,21 +389,16 @@ ProcXFixesGetCursorImage(ClientPtr client)
     };
 
     if (client->swapped) {
-        swaps(&rep.sequenceNumber);
-        swapl(&rep.length);
-        swaps(&rep.x);
-        swaps(&rep.y);
-        swaps(&rep.width);
-        swaps(&rep.height);
-        swaps(&rep.xhot);
-        swaps(&rep.yhot);
-        swapl(&rep.cursorSerial);
-        SwapLongs(image, npixels);
+        swaps(&reply.x);
+        swaps(&reply.y);
+        swaps(&reply.width);
+        swaps(&reply.height);
+        swaps(&reply.xhot);
+        swaps(&reply.yhot);
+        swapl(&reply.cursorSerial);
     }
-    WriteToClient(client, sizeof(rep), &rep);
-    WriteToClient(client, npixels * sizeof(CARD32), image);
-    free(image);
-    return Success;
+
+    return X_SEND_REPLY_WITH_RPCBUF(client, reply, rpcbuf);
 }
 
 int
@@ -416,6 +411,12 @@ ProcXFixesSetCursorName(ClientPtr client)
     Atom atom;
 
     REQUEST_FIXED_SIZE(xXFixesSetCursorNameReq, stuff->nbytes);
+
+    if (client->swapped) {
+        swapl(&stuff->cursor);
+        swaps(&stuff->nbytes);
+    }
+
     VERIFY_CURSOR(pCursor, stuff->cursor, client, DixSetAttrAccess);
     tchar = (char *) &stuff[1];
     atom = MakeAtom(tchar, stuff->nbytes, TRUE);
@@ -426,16 +427,6 @@ ProcXFixesSetCursorName(ClientPtr client)
     return Success;
 }
 
-int _X_COLD
-SProcXFixesSetCursorName(ClientPtr client)
-{
-    REQUEST(xXFixesSetCursorNameReq);
-    REQUEST_AT_LEAST_SIZE(xXFixesSetCursorNameReq);
-    swapl(&stuff->cursor);
-    swaps(&stuff->nbytes);
-    return ProcXFixesSetCursorName(client);
-}
-
 int
 ProcXFixesGetCursorName(ClientPtr client)
 {
@@ -443,42 +434,31 @@ ProcXFixesGetCursorName(ClientPtr client)
 
     CursorPtr pCursor;
     const char *str;
-    int len;
 
     REQUEST_SIZE_MATCH(xXFixesGetCursorNameReq);
+
+    if (client->swapped)
+        swapl(&stuff->cursor);
+
     VERIFY_CURSOR(pCursor, stuff->cursor, client, DixGetAttrAccess);
     if (pCursor->name)
         str = NameForAtom(pCursor->name);
     else
         str = "";
-    len = strlen(str);
 
-    xXFixesGetCursorNameReply rep = {
-        .type = X_Reply,
-        .sequenceNumber = client->sequence,
-        .length = bytes_to_int32(len),
+    x_rpcbuf_t rpcbuf = { .swapped = client->swapped, .err_clear = TRUE };
+    x_rpcbuf_write_string_pad(&rpcbuf, str);
+
+    xXFixesGetCursorNameReply reply = {
         .atom = pCursor->name,
-        .nbytes = len
+        .nbytes = strlen(str)
     };
     if (client->swapped) {
-        swaps(&rep.sequenceNumber);
-        swapl(&rep.length);
-        swapl(&rep.atom);
-        swaps(&rep.nbytes);
+        swapl(&reply.atom);
+        swaps(&reply.nbytes);
     }
-    WriteToClient(client, sizeof(rep), &rep);
-    WriteToClient(client, len, str);
 
-    return Success;
-}
-
-int _X_COLD
-SProcXFixesGetCursorName(ClientPtr client)
-{
-    REQUEST(xXFixesGetCursorNameReq);
-    REQUEST_SIZE_MATCH(xXFixesGetCursorNameReq);
-    swapl(&stuff->cursor);
-    return ProcXFixesGetCursorName(client);
+    return X_SEND_REPLY_WITH_RPCBUF(client, reply, rpcbuf);
 }
 
 int
@@ -488,7 +468,6 @@ ProcXFixesGetCursorImageAndName(ClientPtr client)
     CursorPtr pCursor;
     int npixels;
     const char *name;
-    int nbytes;
     int width, height;
     int rc, x, y;
 
@@ -505,21 +484,23 @@ ProcXFixesGetCursorImageAndName(ClientPtr client)
     height = pCursor->bits->height;
     npixels = width * height;
     name = pCursor->name ? NameForAtom(pCursor->name) : "";
-    nbytes = strlen(name);
 
-    // pixmap plus name (padded to 4 bytes)
-    const size_t image_size = (npixels + bytes_to_int32(nbytes)) * sizeof(CARD32);
-    CARD32 *image = calloc(1, image_size);
+    x_rpcbuf_t rpcbuf = { .swapped = client->swapped, .err_clear = TRUE };
+
+    CARD32 *image = x_rpcbuf_reserve(&rpcbuf, npixels * sizeof(CARD32));
     if (!image)
         return BadAlloc;
 
     CopyCursorToImage(pCursor, image);
-    memcpy((image + npixels), name, nbytes);
+    if (client->swapped)
+        SwapLongs(image, npixels);
 
-    xXFixesGetCursorImageAndNameReply rep = {
-        .type = X_Reply,
-        .sequenceNumber = client->sequence,
-        .length = bytes_to_int32(image_size),
+    x_rpcbuf_write_string_pad(&rpcbuf, name);
+
+    if (rpcbuf.error)
+        return BadAlloc;
+
+    xXFixesGetCursorImageAndNameReply reply = {
         .width = width,
         .height = height,
         .x = x,
@@ -528,27 +509,22 @@ ProcXFixesGetCursorImageAndName(ClientPtr client)
         .yhot = pCursor->bits->yhot,
         .cursorSerial = pCursor->serialNumber,
         .cursorName = pCursor->name,
-        .nbytes = nbytes,
+        .nbytes = strlen(name),
     };
 
     if (client->swapped) {
-        swaps(&rep.sequenceNumber);
-        swapl(&rep.length);
-        swaps(&rep.x);
-        swaps(&rep.y);
-        swaps(&rep.width);
-        swaps(&rep.height);
-        swaps(&rep.xhot);
-        swaps(&rep.yhot);
-        swapl(&rep.cursorSerial);
-        swapl(&rep.cursorName);
-        swaps(&rep.nbytes);
-        SwapLongs(image, npixels);
+        swaps(&reply.x);
+        swaps(&reply.y);
+        swaps(&reply.width);
+        swaps(&reply.height);
+        swaps(&reply.xhot);
+        swaps(&reply.yhot);
+        swapl(&reply.cursorSerial);
+        swapl(&reply.cursorName);
+        swaps(&reply.nbytes);
     }
-    WriteToClient(client, sizeof(rep), &rep);
-    WriteToClient(client, image_size, image);
-    free(image);
-    return Success;
+
+    return X_SEND_REPLY_WITH_RPCBUF(client, reply, rpcbuf);
 }
 
 /*
@@ -644,7 +620,7 @@ ReplaceCursor(CursorPtr pCursor, TestCursorFunc testCursor, void *closure)
         }
     }
     /* this "knows" that WindowHasNewCursor doesn't depend on its argument */
-    WindowHasNewCursor(screenInfo.screens[0]->root);
+    WindowHasNewCursor(dixGetMasterScreen()->root);
 }
 
 static Bool
@@ -659,8 +635,13 @@ ProcXFixesChangeCursor(ClientPtr client)
     CursorPtr pSource, pDestination;
 
     REQUEST(xXFixesChangeCursorReq);
-
     REQUEST_SIZE_MATCH(xXFixesChangeCursorReq);
+
+    if (client->swapped) {
+        swapl(&stuff->source);
+        swapl(&stuff->destination);
+    }
+
     VERIFY_CURSOR(pSource, stuff->source, client,
                   DixReadAccess | DixGetAttrAccess);
     VERIFY_CURSOR(pDestination, stuff->destination, client,
@@ -670,16 +651,6 @@ ProcXFixesChangeCursor(ClientPtr client)
     return Success;
 }
 
-int _X_COLD
-SProcXFixesChangeCursor(ClientPtr client)
-{
-    REQUEST(xXFixesChangeCursorReq);
-    REQUEST_SIZE_MATCH(xXFixesChangeCursorReq);
-    swapl(&stuff->source);
-    swapl(&stuff->destination);
-    return ProcXFixesChangeCursor(client);
-}
-
 static Bool
 TestForCursorName(CursorPtr pCursor, void *closure)
 {
@@ -696,8 +667,13 @@ ProcXFixesChangeCursorByName(ClientPtr client)
     char *tchar;
 
     REQUEST(xXFixesChangeCursorByNameReq);
-
     REQUEST_FIXED_SIZE(xXFixesChangeCursorByNameReq, stuff->nbytes);
+
+    if (client->swapped) {
+        swapl(&stuff->source);
+        swaps(&stuff->nbytes);
+    }
+
     VERIFY_CURSOR(pSource, stuff->source, client,
                   DixReadAccess | DixGetAttrAccess);
     tchar = (char *) &stuff[1];
@@ -707,16 +683,6 @@ ProcXFixesChangeCursorByName(ClientPtr client)
     return Success;
 }
 
-int _X_COLD
-SProcXFixesChangeCursorByName(ClientPtr client)
-{
-    REQUEST(xXFixesChangeCursorByNameReq);
-    REQUEST_AT_LEAST_SIZE(xXFixesChangeCursorByNameReq);
-    swapl(&stuff->source);
-    swaps(&stuff->nbytes);
-    return ProcXFixesChangeCursorByName(client);
-}
-
 /*
  * Routines for manipulating the per-screen hide counts list.
  * This list indicates which clients have requested cursor hiding
@@ -821,6 +787,9 @@ ProcXFixesHideCursor(ClientPtr client)
 
     REQUEST_SIZE_MATCH(xXFixesHideCursorReq);
 
+    if (client->swapped)
+        swapl(&stuff->window);
+
     ret = dixLookupResourceByType((void **) &pWin, stuff->window, X11_RESTYPE_WINDOW,
                                   client, DixGetAttrAccess);
     if (ret != Success) {
@@ -843,7 +812,7 @@ ProcXFixesHideCursor(ClientPtr client)
      * This is the first time this client has hid the cursor
      * for this screen.
      */
-    ret = XaceHookScreenAccess(client, pWin->drawable.pScreen, DixHideAccess);
+    ret = dixCallScreenAccessCallback(client, pWin->drawable.pScreen, DixHideAccess);
     if (ret != Success)
         return ret;
 
@@ -862,15 +831,6 @@ ProcXFixesHideCursor(ClientPtr client)
     return ret;
 }
 
-int _X_COLD
-SProcXFixesHideCursor(ClientPtr client)
-{
-    REQUEST(xXFixesHideCursorReq);
-    REQUEST_SIZE_MATCH(xXFixesHideCursorReq);
-    swapl(&stuff->window);
-    return ProcXFixesHideCursor(client);
-}
-
 int
 ProcXFixesShowCursor(ClientPtr client)
 {
@@ -879,9 +839,11 @@ ProcXFixesShowCursor(ClientPtr client)
     int rc;
 
     REQUEST(xXFixesShowCursorReq);
-
     REQUEST_SIZE_MATCH(xXFixesShowCursorReq);
 
+    if (client->swapped)
+        swapl(&stuff->window);
+
     rc = dixLookupResourceByType((void **) &pWin, stuff->window, X11_RESTYPE_WINDOW,
                                  client, DixGetAttrAccess);
     if (rc != Success) {
@@ -898,7 +860,7 @@ ProcXFixesShowCursor(ClientPtr client)
         return BadMatch;
     }
 
-    rc = XaceHookScreenAccess(client, pWin->drawable.pScreen, DixShowAccess);
+    rc = dixCallScreenAccessCallback(client, pWin->drawable.pScreen, DixShowAccess);
     if (rc != Success)
         return rc;
 
@@ -910,15 +872,6 @@ ProcXFixesShowCursor(ClientPtr client)
     return Success;
 }
 
-int _X_COLD
-SProcXFixesShowCursor(ClientPtr client)
-{
-    REQUEST(xXFixesShowCursorReq);
-    REQUEST_SIZE_MATCH(xXFixesShowCursorReq);
-    swapl(&stuff->window);
-    return ProcXFixesShowCursor(client);
-}
-
 static int
 CursorFreeClient(void *data, XID id)
 {
@@ -971,64 +924,50 @@ ProcXFixesCreatePointerBarrier(ClientPtr client)
 {
     REQUEST(xXFixesCreatePointerBarrierReq);
 
+    if (client->swapped) {
+        REQUEST_AT_LEAST_SIZE(xXFixesCreatePointerBarrierReq);
+
+        swaps(&stuff->num_devices);
+        REQUEST_FIXED_SIZE(xXFixesCreatePointerBarrierReq,
+                           pad_to_int32(stuff->num_devices * sizeof(CARD16)));
+
+        swapl(&stuff->barrier);
+        swapl(&stuff->window);
+        swaps(&stuff->x1);
+        swaps(&stuff->y1);
+        swaps(&stuff->x2);
+        swaps(&stuff->y2);
+        swapl(&stuff->directions);
+
+        CARD16 *in_devices = (CARD16 *) &stuff[1];
+        for (int i = 0; i < stuff->num_devices; i++) {
+            swaps(in_devices + i);
+        }
+    }
+
     REQUEST_FIXED_SIZE(xXFixesCreatePointerBarrierReq,
                        pad_to_int32(stuff->num_devices * sizeof(CARD16)));
+
     LEGAL_NEW_RESOURCE(stuff->barrier, client);
 
     return XICreatePointerBarrier(client, stuff);
 }
 
-int _X_COLD
-SProcXFixesCreatePointerBarrier(ClientPtr client)
-{
-    REQUEST(xXFixesCreatePointerBarrierReq);
-    int i;
-    CARD16 *in_devices = (CARD16 *) &stuff[1];
-
-    REQUEST_AT_LEAST_SIZE(xXFixesCreatePointerBarrierReq);
-
-    swaps(&stuff->num_devices);
-    REQUEST_FIXED_SIZE(xXFixesCreatePointerBarrierReq,
-                       pad_to_int32(stuff->num_devices * sizeof(CARD16)));
-
-    swapl(&stuff->barrier);
-    swapl(&stuff->window);
-    swaps(&stuff->x1);
-    swaps(&stuff->y1);
-    swaps(&stuff->x2);
-    swaps(&stuff->y2);
-    swapl(&stuff->directions);
-    for (i = 0; i < stuff->num_devices; i++) {
-        swaps(in_devices + i);
-    }
-
-    return ProcXFixesCreatePointerBarrier(client);
-}
-
 int
 ProcXFixesDestroyPointerBarrier(ClientPtr client)
 {
     REQUEST(xXFixesDestroyPointerBarrierReq);
-
     REQUEST_SIZE_MATCH(xXFixesDestroyPointerBarrierReq);
 
+    if (client->swapped)
+        swapl(&stuff->barrier);
+
     return XIDestroyPointerBarrier(client, stuff);
 }
 
-int _X_COLD
-SProcXFixesDestroyPointerBarrier(ClientPtr client)
-{
-    REQUEST(xXFixesDestroyPointerBarrierReq);
-    REQUEST_SIZE_MATCH(xXFixesDestroyPointerBarrierReq);
-    swapl(&stuff->barrier);
-    return ProcXFixesDestroyPointerBarrier(client);
-}
-
 Bool
 XFixesCursorInit(void)
 {
-    int i;
-
     if (party_like_its_1989)
         CursorVisible = EnableCursor;
     else
@@ -1037,13 +976,13 @@ XFixesCursorInit(void)
     if (!dixRegisterPrivateKey(&CursorScreenPrivateKeyRec, PRIVATE_SCREEN, sizeof(CursorScreenRec)))
         return FALSE;
 
-    for (i = 0; i < screenInfo.numScreens; i++) {
-        ScreenPtr pScreen = screenInfo.screens[i];
-        CursorScreenPtr cs = GetCursorScreen(pScreen);
-        dixScreenHookClose(pScreen, CursorScreenClose);
-        Wrap(cs, pScreen, DisplayCursor, CursorDisplayCursor);
+    DIX_FOR_EACH_SCREEN({
+        CursorScreenPtr cs = GetCursorScreen(walkScreen);
+        dixScreenHookClose(walkScreen, CursorScreenClose);
+        Wrap(cs, walkScreen, DisplayCursor, CursorDisplayCursor);
         cs->pCursorHideCounts = NULL;
-    }
+    });
+
     CursorClientType = CreateNewResourceType(CursorFreeClient,
                                              "XFixesCursorClient");
     CursorHideCountType = CreateNewResourceType(CursorFreeHideCount,
