@@ -23,6 +23,8 @@
 #include <dix-config.h>
 
 #include "dix/dix_priv.h"
+#include "dix/request_priv.h"
+#include "dix/rpcbuf_priv.h"
 #include "dix/window_priv.h"
 #include "render/picturestr_priv.h"
 #include "Xext/panoramiX.h"
@@ -76,8 +78,13 @@ ProcXFixesCreateRegion(ClientPtr client)
     RegionPtr pRegion;
 
     REQUEST(xXFixesCreateRegionReq);
-
     REQUEST_AT_LEAST_SIZE(xXFixesCreateRegionReq);
+
+    if (client->swapped) {
+        swapl(&stuff->region);
+        SwapRestS(stuff);
+    }
+
     LEGAL_NEW_RESOURCE(stuff->region, client);
 
     things = (client->req_len << 2) - sizeof(xXFixesCreateRegionReq);
@@ -94,16 +101,6 @@ ProcXFixesCreateRegion(ClientPtr client)
     return Success;
 }
 
-int _X_COLD
-SProcXFixesCreateRegion(ClientPtr client)
-{
-    REQUEST(xXFixesCreateRegionReq);
-    REQUEST_AT_LEAST_SIZE(xXFixesCreateRegionReq);
-    swapl(&stuff->region);
-    SwapRestS(stuff);
-    return ProcXFixesCreateRegion(client);
-}
-
 int
 ProcXFixesCreateRegionFromBitmap(ClientPtr client)
 {
@@ -112,8 +109,13 @@ ProcXFixesCreateRegionFromBitmap(ClientPtr client)
     int rc;
 
     REQUEST(xXFixesCreateRegionFromBitmapReq);
-
     REQUEST_SIZE_MATCH(xXFixesCreateRegionFromBitmapReq);
+
+    if (client->swapped) {
+        swapl(&stuff->region);
+        swapl(&stuff->bitmap);
+    }
+
     LEGAL_NEW_RESOURCE(stuff->region, client);
 
     rc = dixLookupResourceByType((void **) &pPixmap, stuff->bitmap, X11_RESTYPE_PIXMAP,
@@ -136,16 +138,6 @@ ProcXFixesCreateRegionFromBitmap(ClientPtr client)
     return Success;
 }
 
-int _X_COLD
-SProcXFixesCreateRegionFromBitmap(ClientPtr client)
-{
-    REQUEST(xXFixesCreateRegionFromBitmapReq);
-    REQUEST_SIZE_MATCH(xXFixesCreateRegionFromBitmapReq);
-    swapl(&stuff->region);
-    swapl(&stuff->bitmap);
-    return ProcXFixesCreateRegionFromBitmap(client);
-}
-
 int
 ProcXFixesCreateRegionFromWindow(ClientPtr client)
 {
@@ -155,8 +147,13 @@ ProcXFixesCreateRegionFromWindow(ClientPtr client)
     int rc;
 
     REQUEST(xXFixesCreateRegionFromWindowReq);
-
     REQUEST_SIZE_MATCH(xXFixesCreateRegionFromWindowReq);
+
+    if (client->swapped) {
+        swapl(&stuff->region);
+        swapl(&stuff->window);
+    }
+
     LEGAL_NEW_RESOURCE(stuff->region, client);
     rc = dixLookupResourceByType((void **) &pWin, stuff->window, X11_RESTYPE_WINDOW,
                                  client, DixGetAttrAccess);
@@ -193,16 +190,6 @@ ProcXFixesCreateRegionFromWindow(ClientPtr client)
     return Success;
 }
 
-int _X_COLD
-SProcXFixesCreateRegionFromWindow(ClientPtr client)
-{
-    REQUEST(xXFixesCreateRegionFromWindowReq);
-    REQUEST_SIZE_MATCH(xXFixesCreateRegionFromWindowReq);
-    swapl(&stuff->region);
-    swapl(&stuff->window);
-    return ProcXFixesCreateRegionFromWindow(client);
-}
-
 int
 ProcXFixesCreateRegionFromGC(ClientPtr client)
 {
@@ -211,8 +198,13 @@ ProcXFixesCreateRegionFromGC(ClientPtr client)
     int rc;
 
     REQUEST(xXFixesCreateRegionFromGCReq);
-
     REQUEST_SIZE_MATCH(xXFixesCreateRegionFromGCReq);
+
+    if (client->swapped) {
+        swapl(&stuff->region);
+        swapl(&stuff->gc);
+    }
+
     LEGAL_NEW_RESOURCE(stuff->region, client);
 
     rc = dixLookupGC(&pGC, stuff->gc, client, DixGetAttrAccess);
@@ -234,16 +226,6 @@ ProcXFixesCreateRegionFromGC(ClientPtr client)
     return Success;
 }
 
-int _X_COLD
-SProcXFixesCreateRegionFromGC(ClientPtr client)
-{
-    REQUEST(xXFixesCreateRegionFromGCReq);
-    REQUEST_SIZE_MATCH(xXFixesCreateRegionFromGCReq);
-    swapl(&stuff->region);
-    swapl(&stuff->gc);
-    return ProcXFixesCreateRegionFromGC(client);
-}
-
 int
 ProcXFixesCreateRegionFromPicture(ClientPtr client)
 {
@@ -251,8 +233,13 @@ ProcXFixesCreateRegionFromPicture(ClientPtr client)
     PicturePtr pPicture;
 
     REQUEST(xXFixesCreateRegionFromPictureReq);
-
     REQUEST_SIZE_MATCH(xXFixesCreateRegionFromPictureReq);
+
+    if (client->swapped) {
+        swapl(&stuff->region);
+        swapl(&stuff->picture);
+    }
+
     LEGAL_NEW_RESOURCE(stuff->region, client);
 
     VERIFY_PICTURE(pPicture, stuff->picture, client, DixGetAttrAccess);
@@ -274,16 +261,6 @@ ProcXFixesCreateRegionFromPicture(ClientPtr client)
     return Success;
 }
 
-int _X_COLD
-SProcXFixesCreateRegionFromPicture(ClientPtr client)
-{
-    REQUEST(xXFixesCreateRegionFromPictureReq);
-    REQUEST_SIZE_MATCH(xXFixesCreateRegionFromPictureReq);
-    swapl(&stuff->region);
-    swapl(&stuff->picture);
-    return ProcXFixesCreateRegionFromPicture(client);
-}
-
 int
 ProcXFixesDestroyRegion(ClientPtr client)
 {
@@ -291,20 +268,15 @@ ProcXFixesDestroyRegion(ClientPtr client)
     RegionPtr pRegion;
 
     REQUEST_SIZE_MATCH(xXFixesDestroyRegionReq);
+
+    if (client->swapped)
+        swapl(&stuff->region);
+
     VERIFY_REGION(pRegion, stuff->region, client, DixWriteAccess);
     FreeResource(stuff->region, X11_RESTYPE_NONE);
     return Success;
 }
 
-int _X_COLD
-SProcXFixesDestroyRegion(ClientPtr client)
-{
-    REQUEST(xXFixesDestroyRegionReq);
-    REQUEST_SIZE_MATCH(xXFixesDestroyRegionReq);
-    swapl(&stuff->region);
-    return ProcXFixesDestroyRegion(client);
-}
-
 int
 ProcXFixesSetRegion(ClientPtr client)
 {
@@ -312,8 +284,13 @@ ProcXFixesSetRegion(ClientPtr client)
     RegionPtr pRegion, pNew;
 
     REQUEST(xXFixesSetRegionReq);
-
     REQUEST_AT_LEAST_SIZE(xXFixesSetRegionReq);
+
+    if (client->swapped) {
+        swapl(&stuff->region);
+        SwapRestS(stuff);
+    }
+
     VERIFY_REGION(pRegion, stuff->region, client, DixWriteAccess);
 
     things = (client->req_len << 2) - sizeof(xXFixesCreateRegionReq);
@@ -332,16 +309,6 @@ ProcXFixesSetRegion(ClientPtr client)
     return Success;
 }
 
-int _X_COLD
-SProcXFixesSetRegion(ClientPtr client)
-{
-    REQUEST(xXFixesSetRegionReq);
-    REQUEST_AT_LEAST_SIZE(xXFixesSetRegionReq);
-    swapl(&stuff->region);
-    SwapRestS(stuff);
-    return ProcXFixesSetRegion(client);
-}
-
 int
 ProcXFixesCopyRegion(ClientPtr client)
 {
@@ -350,6 +317,11 @@ ProcXFixesCopyRegion(ClientPtr client)
     REQUEST(xXFixesCopyRegionReq);
     REQUEST_SIZE_MATCH(xXFixesCopyRegionReq);
 
+    if (client->swapped) {
+        swapl(&stuff->source);
+        swapl(&stuff->destination);
+    }
+
     VERIFY_REGION(pSource, stuff->source, client, DixReadAccess);
     VERIFY_REGION(pDestination, stuff->destination, client, DixWriteAccess);
 
@@ -359,24 +331,20 @@ ProcXFixesCopyRegion(ClientPtr client)
     return Success;
 }
 
-int _X_COLD
-SProcXFixesCopyRegion(ClientPtr client)
-{
-    REQUEST(xXFixesCopyRegionReq);
-    REQUEST_SIZE_MATCH(xXFixesCopyRegionReq);
-    swapl(&stuff->source);
-    swapl(&stuff->destination);
-    return ProcXFixesCopyRegion(client);
-}
-
 int
 ProcXFixesCombineRegion(ClientPtr client)
 {
     RegionPtr pSource1, pSource2, pDestination;
 
     REQUEST(xXFixesCombineRegionReq);
-
     REQUEST_SIZE_MATCH(xXFixesCombineRegionReq);
+
+    if (client->swapped) {
+        swapl(&stuff->source1);
+        swapl(&stuff->source2);
+        swapl(&stuff->destination);
+    }
+
     VERIFY_REGION(pSource1, stuff->source1, client, DixReadAccess);
     VERIFY_REGION(pSource2, stuff->source2, client, DixReadAccess);
     VERIFY_REGION(pDestination, stuff->destination, client, DixWriteAccess);
@@ -399,17 +367,6 @@ ProcXFixesCombineRegion(ClientPtr client)
     return Success;
 }
 
-int _X_COLD
-SProcXFixesCombineRegion(ClientPtr client)
-{
-    REQUEST(xXFixesCombineRegionReq);
-    REQUEST_SIZE_MATCH(xXFixesCombineRegionReq);
-    swapl(&stuff->source1);
-    swapl(&stuff->source2);
-    swapl(&stuff->destination);
-    return ProcXFixesCombineRegion(client);
-}
-
 int
 ProcXFixesInvertRegion(ClientPtr client)
 {
@@ -417,8 +374,17 @@ ProcXFixesInvertRegion(ClientPtr client)
     BoxRec bounds;
 
     REQUEST(xXFixesInvertRegionReq);
-
     REQUEST_SIZE_MATCH(xXFixesInvertRegionReq);
+
+    if (client->swapped) {
+        swapl(&stuff->source);
+        swaps(&stuff->x);
+        swaps(&stuff->y);
+        swaps(&stuff->width);
+        swaps(&stuff->height);
+        swapl(&stuff->destination);
+    }
+
     VERIFY_REGION(pSource, stuff->source, client, DixReadAccess);
     VERIFY_REGION(pDestination, stuff->destination, client, DixWriteAccess);
 
@@ -441,53 +407,39 @@ ProcXFixesInvertRegion(ClientPtr client)
     return Success;
 }
 
-int _X_COLD
-SProcXFixesInvertRegion(ClientPtr client)
-{
-    REQUEST(xXFixesInvertRegionReq);
-    REQUEST_SIZE_MATCH(xXFixesInvertRegionReq);
-    swapl(&stuff->source);
-    swaps(&stuff->x);
-    swaps(&stuff->y);
-    swaps(&stuff->width);
-    swaps(&stuff->height);
-    swapl(&stuff->destination);
-    return ProcXFixesInvertRegion(client);
-}
-
 int
 ProcXFixesTranslateRegion(ClientPtr client)
 {
     RegionPtr pRegion;
 
     REQUEST(xXFixesTranslateRegionReq);
-
     REQUEST_SIZE_MATCH(xXFixesTranslateRegionReq);
+
+    if (client->swapped) {
+        swapl(&stuff->region);
+        swaps(&stuff->dx);
+        swaps(&stuff->dy);
+    }
+
     VERIFY_REGION(pRegion, stuff->region, client, DixWriteAccess);
 
     RegionTranslate(pRegion, stuff->dx, stuff->dy);
     return Success;
 }
 
-int _X_COLD
-SProcXFixesTranslateRegion(ClientPtr client)
-{
-    REQUEST(xXFixesTranslateRegionReq);
-    REQUEST_SIZE_MATCH(xXFixesTranslateRegionReq);
-    swapl(&stuff->region);
-    swaps(&stuff->dx);
-    swaps(&stuff->dy);
-    return ProcXFixesTranslateRegion(client);
-}
-
 int
 ProcXFixesRegionExtents(ClientPtr client)
 {
     RegionPtr pSource, pDestination;
 
     REQUEST(xXFixesRegionExtentsReq);
-
     REQUEST_SIZE_MATCH(xXFixesRegionExtentsReq);
+
+    if (client->swapped) {
+        swapl(&stuff->source);
+        swapl(&stuff->destination);
+    }
+
     VERIFY_REGION(pSource, stuff->source, client, DixReadAccess);
     VERIFY_REGION(pDestination, stuff->destination, client, DixWriteAccess);
 
@@ -496,16 +448,6 @@ ProcXFixesRegionExtents(ClientPtr client)
     return Success;
 }
 
-int _X_COLD
-SProcXFixesRegionExtents(ClientPtr client)
-{
-    REQUEST(xXFixesRegionExtentsReq);
-    REQUEST_SIZE_MATCH(xXFixesRegionExtentsReq);
-    swapl(&stuff->source);
-    swapl(&stuff->destination);
-    return ProcXFixesRegionExtents(client);
-}
-
 int
 ProcXFixesFetchRegion(ClientPtr client)
 {
@@ -515,29 +457,29 @@ ProcXFixesFetchRegion(ClientPtr client)
     int i, nBox;
 
     REQUEST(xXFixesFetchRegionReq);
-
     REQUEST_SIZE_MATCH(xXFixesFetchRegionReq);
+
+    if (client->swapped) {
+        swapl(&stuff->region);
+    }
+
     VERIFY_REGION(pRegion, stuff->region, client, DixReadAccess);
 
     pExtent = RegionExtents(pRegion);
     pBox = RegionRects(pRegion);
     nBox = RegionNumRects(pRegion);
 
-    xRectangle *pRect = calloc(nBox, sizeof(xRectangle));
-    if (!pRect)
-        return BadAlloc;
+    x_rpcbuf_t rpcbuf = { .swapped = client->swapped, .err_clear = TRUE };
 
     for (i = 0; i < nBox; i++) {
-        pRect[i].x = pBox[i].x1;
-        pRect[i].y = pBox[i].y1;
-        pRect[i].width = pBox[i].x2 - pBox[i].x1;
-        pRect[i].height = pBox[i].y2 - pBox[i].y1;
+        x_rpcbuf_write_rect(&rpcbuf,
+                            pBox[i].x1,
+                            pBox[i].y1,
+                            pBox[i].x2 - pBox[i].x1,
+                            pBox[i].y2 - pBox[i].y1);
     }
 
-    xXFixesFetchRegionReply rep = {
-        .type = X_Reply,
-        .sequenceNumber = client->sequence,
-        .length = nBox << 1,
+    xXFixesFetchRegionReply reply = {
         .x = pExtent->x1,
         .y = pExtent->y1,
         .width = pExtent->x2 - pExtent->x1,
@@ -545,31 +487,21 @@ ProcXFixesFetchRegion(ClientPtr client)
     };
 
     if (client->swapped) {
-        swaps(&rep.sequenceNumber);
-        swapl(&rep.length);
-        swaps(&rep.x);
-        swaps(&rep.y);
-        swaps(&rep.width);
-        swaps(&rep.height);
-        SwapShorts((INT16 *) pRect, nBox * 4);
+        swaps(&reply.sequenceNumber);
+        swapl(&reply.length);
+        swaps(&reply.x);
+        swaps(&reply.y);
+        swaps(&reply.width);
+        swaps(&reply.height);
     }
-    WriteToClient(client, sizeof(rep), &rep);
-    WriteToClient(client, nBox * sizeof(xRectangle), pRect);
-    free(pRect);
-    return Success;
-}
 
-int _X_COLD
-SProcXFixesFetchRegion(ClientPtr client)
-{
-    REQUEST(xXFixesFetchRegionReq);
-    REQUEST_SIZE_MATCH(xXFixesFetchRegionReq);
-    swapl(&stuff->region);
-    return ProcXFixesFetchRegion(client);
+    return X_SEND_REPLY_WITH_RPCBUF(client, reply, rpcbuf);
 }
 
+#ifdef XINERAMA
 static int
 PanoramiXFixesSetGCClipRegion(ClientPtr client, xXFixesSetGCClipRegionReq *stuff);
+#endif
 
 static int
 SingleXFixesSetGCClipRegion(ClientPtr client, xXFixesSetGCClipRegionReq *stuff);
@@ -580,6 +512,13 @@ ProcXFixesSetGCClipRegion(ClientPtr client)
     REQUEST(xXFixesSetGCClipRegionReq);
     REQUEST_SIZE_MATCH(xXFixesSetGCClipRegionReq);
 
+    if (client->swapped) {
+        swapl(&stuff->gc);
+        swapl(&stuff->region);
+        swaps(&stuff->xOrigin);
+        swaps(&stuff->yOrigin);
+    }
+
 #ifdef XINERAMA
     if (XFixesUseXinerama)
         return PanoramiXFixesSetGCClipRegion(client, stuff);
@@ -609,25 +548,13 @@ SingleXFixesSetGCClipRegion(ClientPtr client, xXFixesSetGCClipRegionReq *stuff)
 
     vals[0].val = stuff->xOrigin;
     vals[1].val = stuff->yOrigin;
-    ChangeGC(NullClient, pGC, GCClipXOrigin | GCClipYOrigin, vals);
+    ChangeGC(NULL, pGC, GCClipXOrigin | GCClipYOrigin, vals);
     (*pGC->funcs->ChangeClip) (pGC, pRegion ? CT_REGION : CT_NONE,
                                (void *) pRegion, 0);
 
     return Success;
 }
 
-int _X_COLD
-SProcXFixesSetGCClipRegion(ClientPtr client)
-{
-    REQUEST(xXFixesSetGCClipRegionReq);
-    REQUEST_SIZE_MATCH(xXFixesSetGCClipRegionReq);
-    swapl(&stuff->gc);
-    swapl(&stuff->region);
-    swaps(&stuff->xOrigin);
-    swaps(&stuff->yOrigin);
-    return ProcXFixesSetGCClipRegion(client);
-}
-
 typedef RegionPtr (*CreateDftPtr) (WindowPtr pWin);
 
 static int
@@ -701,8 +628,10 @@ SingleXFixesSetWindowShapeRegion(ClientPtr client, xXFixesSetWindowShapeRegionRe
     return Success;
 }
 
+#ifdef XINERAMA
 static int
 PanoramiXFixesSetWindowShapeRegion(ClientPtr client, xXFixesSetWindowShapeRegionReq *stuff);
+#endif
 
 int
 ProcXFixesSetWindowShapeRegion(ClientPtr client)
@@ -710,6 +639,13 @@ ProcXFixesSetWindowShapeRegion(ClientPtr client)
     REQUEST(xXFixesSetWindowShapeRegionReq);
     REQUEST_SIZE_MATCH(xXFixesSetWindowShapeRegionReq);
 
+    if (client->swapped) {
+        swapl(&stuff->dest);
+        swaps(&stuff->xOff);
+        swaps(&stuff->yOff);
+        swapl(&stuff->region);
+    }
+
 #ifdef XINERAMA
     if (XFixesUseXinerama)
         return PanoramiXFixesSetWindowShapeRegion(client, stuff);
@@ -717,23 +653,13 @@ ProcXFixesSetWindowShapeRegion(ClientPtr client)
     return SingleXFixesSetWindowShapeRegion(client, stuff);
 }
 
-int _X_COLD
-SProcXFixesSetWindowShapeRegion(ClientPtr client)
-{
-    REQUEST(xXFixesSetWindowShapeRegionReq);
-    REQUEST_SIZE_MATCH(xXFixesSetWindowShapeRegionReq);
-    swapl(&stuff->dest);
-    swaps(&stuff->xOff);
-    swaps(&stuff->yOff);
-    swapl(&stuff->region);
-    return ProcXFixesSetWindowShapeRegion(client);
-}
-
 static int
 SingleXFixesSetPictureClipRegion(ClientPtr client, xXFixesSetPictureClipRegionReq *stuff);
 
+#ifdef XINERAMA
 static int
 PanoramiXFixesSetPictureClipRegion(ClientPtr client, xXFixesSetPictureClipRegionReq *stuff);
+#endif
 
 int
 ProcXFixesSetPictureClipRegion(ClientPtr client)
@@ -741,6 +667,13 @@ ProcXFixesSetPictureClipRegion(ClientPtr client)
     REQUEST(xXFixesSetPictureClipRegionReq);
     REQUEST_SIZE_MATCH(xXFixesSetPictureClipRegionReq);
 
+    if (client->swapped) {
+        swapl(&stuff->picture);
+        swapl(&stuff->region);
+        swaps(&stuff->xOrigin);
+        swaps(&stuff->yOrigin);
+    }
+
 #ifdef XINERAMA
     if (XFixesUseXinerama)
         return PanoramiXFixesSetPictureClipRegion(client, stuff);
@@ -764,18 +697,6 @@ SingleXFixesSetPictureClipRegion(ClientPtr client, xXFixesSetPictureClipRegionRe
                                 pRegion);
 }
 
-int _X_COLD
-SProcXFixesSetPictureClipRegion(ClientPtr client)
-{
-    REQUEST(xXFixesSetPictureClipRegionReq);
-    REQUEST_SIZE_MATCH(xXFixesSetPictureClipRegionReq);
-    swapl(&stuff->picture);
-    swapl(&stuff->region);
-    swaps(&stuff->xOrigin);
-    swaps(&stuff->yOrigin);
-    return ProcXFixesSetPictureClipRegion(client);
-}
-
 int
 ProcXFixesExpandRegion(ClientPtr client)
 {
@@ -788,6 +709,16 @@ ProcXFixesExpandRegion(ClientPtr client)
     int i;
 
     REQUEST_SIZE_MATCH(xXFixesExpandRegionReq);
+
+    if (client->swapped) {
+        swapl(&stuff->source);
+        swapl(&stuff->destination);
+        swaps(&stuff->left);
+        swaps(&stuff->right);
+        swaps(&stuff->top);
+        swaps(&stuff->bottom);
+    }
+
     VERIFY_REGION(pSource, stuff->source, client, DixReadAccess);
     VERIFY_REGION(pDestination, stuff->destination, client, DixWriteAccess);
 
@@ -815,26 +746,12 @@ ProcXFixesExpandRegion(ClientPtr client)
     return Success;
 }
 
-int _X_COLD
-SProcXFixesExpandRegion(ClientPtr client)
-{
-    REQUEST(xXFixesExpandRegionReq);
-    REQUEST_SIZE_MATCH(xXFixesExpandRegionReq);
-    swapl(&stuff->source);
-    swapl(&stuff->destination);
-    swaps(&stuff->left);
-    swaps(&stuff->right);
-    swaps(&stuff->top);
-    swaps(&stuff->bottom);
-    return ProcXFixesExpandRegion(client);
-}
-
 #ifdef XINERAMA
 
 static int
 PanoramiXFixesSetGCClipRegion(ClientPtr client, xXFixesSetGCClipRegionReq *stuff)
 {
-    int result = Success, j;
+    int result = Success;
     PanoramiXRes *gc;
 
     if ((result = dixLookupResourceByType((void **) &gc, stuff->gc, XRT_GC,
@@ -843,12 +760,12 @@ PanoramiXFixesSetGCClipRegion(ClientPtr client, xXFixesSetGCClipRegionReq *stuff
         return result;
     }
 
-    FOR_NSCREENS_BACKWARD(j) {
-        stuff->gc = gc->info[j].id;
+    XINERAMA_FOR_EACH_SCREEN_BACKWARD({
+        stuff->gc = gc->info[walkScreenIdx].id;
         result = SingleXFixesSetGCClipRegion(client, stuff);
         if (result != Success)
             break;
-    }
+    });
 
     return result;
 }
@@ -856,7 +773,7 @@ PanoramiXFixesSetGCClipRegion(ClientPtr client, xXFixesSetGCClipRegionReq *stuff
 static int
 PanoramiXFixesSetWindowShapeRegion(ClientPtr client, xXFixesSetWindowShapeRegionReq *stuff)
 {
-    int result = Success, j;
+    int result = Success;
     PanoramiXRes *win;
     RegionPtr reg = NULL;
 
@@ -870,21 +787,20 @@ PanoramiXFixesSetWindowShapeRegion(ClientPtr client, xXFixesSetWindowShapeRegion
     if (win->u.win.root)
         VERIFY_REGION_OR_NONE(reg, stuff->region, client, DixReadAccess);
 
-    FOR_NSCREENS_FORWARD(j) {
-        ScreenPtr screen = screenInfo.screens[j];
-        stuff->dest = win->info[j].id;
+    XINERAMA_FOR_EACH_SCREEN_FORWARD({
+        stuff->dest = win->info[walkScreenIdx].id;
 
         if (reg)
-            RegionTranslate(reg, -screen->x, -screen->y);
+            RegionTranslate(reg, -walkScreen->x, -walkScreen->y);
 
         result = SingleXFixesSetWindowShapeRegion(client, stuff);
 
         if (reg)
-            RegionTranslate(reg, screen->x, screen->y);
+            RegionTranslate(reg, walkScreen->x, walkScreen->y);
 
         if (result != Success)
             break;
-    }
+    });
 
     return result;
 }
@@ -892,7 +808,7 @@ PanoramiXFixesSetWindowShapeRegion(ClientPtr client, xXFixesSetWindowShapeRegion
 static int
 PanoramiXFixesSetPictureClipRegion(ClientPtr client, xXFixesSetPictureClipRegionReq *stuff)
 {
-    int result = Success, j;
+    int result = Success;
     PanoramiXRes *pict;
     RegionPtr reg = NULL;
 
@@ -906,21 +822,20 @@ PanoramiXFixesSetPictureClipRegion(ClientPtr client, xXFixesSetPictureClipRegion
     if (pict->u.pict.root)
         VERIFY_REGION_OR_NONE(reg, stuff->region, client, DixReadAccess);
 
-    FOR_NSCREENS_BACKWARD(j) {
-        ScreenPtr screen = screenInfo.screens[j];
-        stuff->picture = pict->info[j].id;
+    XINERAMA_FOR_EACH_SCREEN_BACKWARD({
+        stuff->picture = pict->info[walkScreenIdx].id;
 
         if (reg)
-            RegionTranslate(reg, -screen->x, -screen->y);
+            RegionTranslate(reg, -walkScreen->x, -walkScreen->y);
 
         result = SingleXFixesSetPictureClipRegion(client, stuff);
 
         if (reg)
-            RegionTranslate(reg, screen->x, screen->y);
+            RegionTranslate(reg, walkScreen->x, walkScreen->y);
 
         if (result != Success)
             break;
-    }
+    });
 
     return result;
 }
